@@ -36,18 +36,18 @@ G_BEGIN_DECLS
 #define EPHY_IS_EMBED_CLASS(k)        (G_TYPE_CHECK_CLASS_TYPE ((k), EPHY_TYPE_EMBED))
 #define EPHY_EMBED_GET_CLASS(o)       (G_TYPE_INSTANCE_GET_CLASS ((o), EPHY_TYPE_EMBED, EphyEmbedClass))
 
-typedef struct EphyEmbedClass EphyEmbedClass;
-typedef struct EphyEmbed EphyEmbed;
-typedef struct EphyEmbedPrivate EphyEmbedPrivate;
+typedef struct _EphyEmbedClass EphyEmbedClass;
+typedef struct _EphyEmbed EphyEmbed;
+typedef struct _EphyEmbedPrivate EphyEmbedPrivate;
 
-struct EphyEmbed {
+struct _EphyEmbed {
   GtkVBox parent_instance;
 
   /*< private >*/
   EphyEmbedPrivate *priv;
 };
 
-struct EphyEmbedClass {
+struct _EphyEmbedClass {
   GtkVBoxClass parent_class;
 };
 
@@ -56,6 +56,7 @@ EphyWebView* ephy_embed_get_web_view      (EphyEmbed *embed);
 void         ephy_embed_add_top_widget    (EphyEmbed *embed, GtkWidget *widget, gboolean destroy_on_transition);
 void         ephy_embed_remove_top_widget (EphyEmbed *embed, GtkWidget *widget);
 void         ephy_embed_auto_download_url (EphyEmbed *embed, const char *url);
+void         _ephy_embed_set_statusbar_label (EphyEmbed *embed, const char *label);
 
 G_END_DECLS
 
